@@ -38,18 +38,18 @@ namespace dd {
     static constexpr std::uint_fast8_t NEDGE = RADIX * RADIX;
 
     enum class BasisStates {
-        zero,
-        one,
-        plus,
-        minus,
-        right,
-        left
+        zero,  // NOLINT(readability-identifier-naming)
+        one,   // NOLINT(readability-identifier-naming)
+        plus,  // NOLINT(readability-identifier-naming)
+        minus, // NOLINT(readability-identifier-naming)
+        right, // NOLINT(readability-identifier-naming)
+        left   // NOLINT(readability-identifier-naming)
     };
 
-    static constexpr fp SQRT2_2 = 0.707106781186547524400844362104849039284835937688474036588L;
-    static constexpr fp PI      = 3.141592653589793238462643383279502884197169399375105820974L;
-    static constexpr fp PI_2    = 1.570796326794896619231321691639751442098584699687552910487L;
-    static constexpr fp PI_4    = 0.785398163397448309615660845819875721049292349843776455243L;
+    static constexpr fp SQRT2_2 = static_cast<fp>(0.707106781186547524400844362104849039284835937688474036588L);
+    static constexpr fp PI      = static_cast<fp>(3.141592653589793238462643383279502884197169399375105820974L);
+    static constexpr fp PI_2    = static_cast<fp>(1.570796326794896619231321691639751442098584699687552910487L);
+    static constexpr fp PI_4    = static_cast<fp>(0.785398163397448309615660845819875721049292349843776455243L);
 
     using CVec = std::vector<std::complex<fp>>;
     using CMat = std::vector<CVec>;
@@ -88,11 +88,14 @@ namespace dd {
 
     // calculates the Units in Last Place (ULP) distance of two floating point numbers
     [[maybe_unused]] static std::size_t ulpDistance(fp a, fp b) {
-        if (a == b)
+        // NOLINTNEXTLINE(clang-diagnostic-float-equal)
+        if (a == b) {
             return 0;
+        }
 
         std::size_t ulps   = 1;
         fp          nextFP = std::nextafter(a, b);
+        // NOLINTNEXTLINE(clang-diagnostic-float-equal)
         while (nextFP != b) {
             ulps++;
             nextFP = std::nextafter(nextFP, b);
