@@ -8,7 +8,7 @@
 
 #include "ComplexNumbers.hpp"
 #include "Definitions.hpp"
-#include "Edge.hpp"
+#include "Node.hpp"
 
 #include <algorithm>
 #include <array>
@@ -33,14 +33,8 @@ namespace dd {
     template<class Node, std::size_t NBUCKET = 32768, std::size_t INITIAL_ALLOCATION_SIZE = 2048, std::size_t GROWTH_FACTOR = 2, std::size_t INITIAL_GC_LIMIT = 131072>
     class UniqueTable {
     public:
-        explicit UniqueTable(std::size_t nvars):
-            nvars(nvars), chunkID(0), allocationSize(INITIAL_ALLOCATION_SIZE), gcLimit(INITIAL_GC_LIMIT) {
-            // allocate first chunk of nodes
-            chunks.emplace_back(allocationSize);
-            allocations += allocationSize;
-            allocationSize *= GROWTH_FACTOR;
-            chunkIt    = chunks[0].begin();
-            chunkEndIt = chunks[0].end();
+        explicit UniqueTable(const std::size_t nv):
+            nvars(nv) {
         }
 
         ~UniqueTable() = default;
@@ -81,48 +75,40 @@ namespace dd {
 
         [[nodiscard]] const auto& getTables() const { return tables; }
 
+        static bool nodesAreEqual(const Node* p, const Node* q) {
+            if constexpr (std::is_same_v<Node, dNode>) {
+                return (p->e == q->e && (p->flags == q->flags));
+            } else {
+                return p->e == q->e;
+            }
+        }
+
         // lookup a node in the unique table for the appropriate variable; insert it, if it has not been found
         // NOTE: reference counting is to be adjusted by function invoking the table lookup and only normalized nodes shall be stored.
         Edge<Node> lookup(const Edge<Node>& e, bool keepNode = false) {
             // there are unique terminal nodes
-            if (e.isTerminal())
+            if (e.isTerminal()) {
                 return e;
+            }
 
-            lookups++;
+            ++lookups;
             const auto key = hash(e.p);
             const auto v   = e.p->v;
 
             // successors of a node shall either have successive variable numbers or be terminals
-            for ([[maybe_unused]] const auto& edge: e.p->e)
+            for ([[maybe_unused]] const auto& edge: e.p->e) {
                 assert(edge.p->v == v - 1 || edge.isTerminal());
-
-            Node* p = tables[v][key];
-            while (p != nullptr) {
-                if (e.p->e == p->e) {
-                    // Match found
-                    if (e.p != p && !keepNode) {
-                        // put node pointed to by e.p on available chain
-                        returnNode(e.p);
-                    }
-                    hits++;
-
-                    // variables should stay the same
-                    assert(p->v == e.p->v);
-
-                    // successors of a node shall either have successive variable numbers or be terminals
-                    for ([[maybe_unused]] const auto& edge: e.p->e)
-                        assert(edge.p->v == v - 1 || edge.isTerminal());
-
-                    return {p, e.w};
-                }
-                collisions++;
-                p = p->next;
             }
 
-            // node was not found -> add it to front of unique table bucket
-            e.p->next      = tables[v][key];
-            tables[v][key] = e.p;
-            nodeCount++;
+            // search bucket in table corresponding to hashed value for the given node and return it if found.
+            if (const auto hashedNode = searchTable(e, key, keepNode); hashedNode != Edge<Node>::zero) {
+                return hashedNode;
+            }
+
+            // if node not found -> add it to front of unique table bucket
+            e.p->next                                = tables[static_cast<std::size_t>(v)][key];
+            tables[static_cast<std::size_t>(v)][key] = e.p;
+            ++nodeCount;
             peakNodeCount = std::max(peakNodeCount, nodeCount);
 
             return e;
@@ -163,8 +149,9 @@ namespace dd {
         // each child if this is the first reference
         void incRef(const Edge<Node>& e) {
             dd::ComplexNumbers::incRef(e.w);
-            if (e.p == nullptr || e.isTerminal())
+            if (e.p == nullptr || e.isTerminal()) {
                 return;
+            }
 
             if (e.p->ref == std::numeric_limits<decltype(e.p->ref)>::max()) {
                 std::clog << "[WARN] MAXREFCNT reached for p=" << reinterpret_cast<std::uintptr_t>(e.p)
@@ -180,7 +167,7 @@ namespace dd {
                         incRef(edge);
                     }
                 }
-                active[e.p->v]++;
+                active[static_cast<std::size_t>(e.p->v)]++;
                 activeNodeCount++;
                 maxActive = std::max(maxActive, activeNodeCount);
             }
@@ -191,8 +178,12 @@ namespace dd {
         // each child if this is the last reference
         void decRef(const Edge<Node>& e) {
             dd::ComplexNumbers::decRef(e.w);
-            if (e.p == nullptr || e.isTerminal()) return;
-            if (e.p->ref == std::numeric_limits<decltype(e.p->ref)>::max()) return;
+            if (e.p == nullptr || e.isTerminal()) {
+                return;
+            }
+            if (e.p->ref == std::numeric_limits<decltype(e.p->ref)>::max()) {
+                return;
+            }
 
             if (e.p->ref == 0) {
                 throw std::runtime_error("In decref: ref==0 before decref\n");
@@ -206,7 +197,7 @@ namespace dd {
                         decRef(edge);
                     }
                 }
-                active[e.p->v]--;
+                active[static_cast<std::size_t>(e.p->v)]--;
                 activeNodeCount--;
             }
         }
@@ -215,8 +206,9 @@ namespace dd {
 
         std::size_t garbageCollect(bool force = false) {
             gcCalls++;
-            if ((!force && nodeCount < gcLimit) || nodeCount == 0)
+            if ((!force && nodeCount < gcLimit) || nodeCount == 0) {
                 return 0;
+            }
 
             gcRuns++;
             std::size_t collected = 0;
@@ -302,15 +294,16 @@ namespace dd {
         };
 
         void print() {
-            Qubit q = nvars - 1;
+            auto q = static_cast<dd::Qubit>(nvars - 1);
             for (auto it = tables.rbegin(); it != tables.rend(); ++it) {
                 auto& table = *it;
                 std::cout << "\tq" << static_cast<std::size_t>(q) << ":"
                           << "\n";
                 for (std::size_t key = 0; key < table.size(); ++key) {
                     auto p = table[key];
-                    if (p != nullptr)
+                    if (p != nullptr) {
                         std::cout << "\tkey=" << key << ": ";
+                    }
 
                     while (p != nullptr) {
                         std::cout << "\t\t" << std::hex << reinterpret_cast<std::uintptr_t>(p) << std::dec << " "
@@ -330,14 +323,15 @@ namespace dd {
 
         void printActive() {
             std::cout << "#printActive: " << activeNodeCount << ", ";
-            for (const auto& a: active)
+            for (const auto& a: active) {
                 std::cout << a << " ";
+            }
             std::cout << "\n";
         }
 
-        [[nodiscard]] fp hitRatio() const { return static_cast<fp>(hits) / lookups; }
+        [[nodiscard]] fp hitRatio() const { return static_cast<fp>(hits) / static_cast<fp>(lookups); }
 
-        [[nodiscard]] fp colRatio() const { return static_cast<fp>(collisions) / lookups; }
+        [[nodiscard]] fp colRatio() const { return static_cast<fp>(collisions) / static_cast<fp>(lookups); }
 
         [[nodiscard]] std::size_t getActiveNodeCount() const {
             return activeNodeCount;
@@ -361,13 +355,13 @@ namespace dd {
         std::vector<Table> tables{nvars};
 
         Node*                                available{};
-        std::vector<std::vector<Node>>       chunks{};
-        std::size_t                          chunkID;
-        typename std::vector<Node>::iterator chunkIt;
-        typename std::vector<Node>::iterator chunkEndIt;
-        std::size_t                          allocationSize;
+        std::vector<std::vector<Node>>       chunks{1, std::vector<Node>{INITIAL_ALLOCATION_SIZE}};
+        std::size_t                          chunkID{0};
+        typename std::vector<Node>::iterator chunkIt{chunks[0].begin()};
+        typename std::vector<Node>::iterator chunkEndIt{chunks[0].end()};
+        std::size_t                          allocationSize{INITIAL_ALLOCATION_SIZE * GROWTH_FACTOR};
 
-        std::size_t allocations   = 0;
+        std::size_t allocations   = INITIAL_ALLOCATION_SIZE;
         std::size_t nodeCount     = 0;
         std::size_t peakNodeCount = 0;
 
@@ -385,7 +379,45 @@ namespace dd {
         // garbage collection
         std::size_t gcCalls = 0;
         std::size_t gcRuns  = 0;
-        std::size_t gcLimit = 250000;
+        std::size_t gcLimit = INITIAL_GC_LIMIT;
+
+        /**
+        Searches for a node in the hash table with the given key.
+        @param e The node to search for.
+        @param key The hashed value used to search the table.
+        @param keepNode If true, the node pointed to by e.p will not be put on the available chain.
+        @return The Edge<Node> found in the hash table or Edge<Node>::zero if not found.
+        **/
+        Edge<Node> searchTable(const Edge<Node>& e, const std::size_t& key, const bool keepNode = false) {
+            const auto v = e.p->v;
+
+            Node* p = tables[static_cast<std::size_t>(v)][key];
+            while (p != nullptr) {
+                if (nodesAreEqual(e.p, p)) {
+                    // Match found
+                    if (e.p != p && !keepNode) {
+                        // put node pointed to by e.p on available chain
+                        returnNode(e.p);
+                    }
+                    ++hits;
+
+                    // variables should stay the same
+                    assert(p->v == e.p->v);
+
+                    // successors of a node shall either have successive variable numbers or be terminals
+                    for ([[maybe_unused]] const auto& edge: e.p->e) {
+                        assert(edge.p->v == v - 1 || edge.isTerminal());
+                    }
+
+                    return {p, e.w};
+                }
+                ++collisions;
+                p = p->next;
+            }
+
+            // Node not found in bucket
+            return Edge<Node>::zero;
+        }
     };
 
 } // namespace dd
