@@ -10,6 +10,7 @@
 #include <fstream>
 #include <iostream>
 #include <map>
+#include <memory>
 #include <sstream>
 #include <stack>
 
@@ -18,18 +19,19 @@ namespace qasm {
     class Scanner {
         struct LineInfo {
             char ch;
-            int  line, col;
+            int  line;
+            int  col;
 
-            LineInfo(char ch, int line, int col):
-                ch(ch), line(line), col(col) {}
+            LineInfo(const char c, const int l, const int cl):
+                ch(c), line(l), col(cl) {}
         };
 
-        std::istream&                      is;
-        std::stack<std::istream*>          streams{};
-        std::map<std::string, Token::Kind> keywords{};
-        char                               ch   = 0;
-        int                                line = 1;
-        int                                col  = 0;
+        std::istream&                                   is;
+        std::stack<std::shared_ptr<std::istream>>       streams{};
+        std::map<std::string, Token::Kind, std::less<>> keywords{};
+        char                                            ch   = 0;
+        int                                             line = 1;
+        int                                             col  = 0;
 
         void nextCh();
 
@@ -44,7 +46,7 @@ namespace qasm {
         std::stack<LineInfo> lines{};
 
     public:
-        explicit Scanner(std::istream& is);
+        explicit Scanner(std::istream& in);
 
         Token next();
 
