@@ -1,16 +1,17 @@
 /*
  * This file is part of MQT QFR library which is released under the MIT license.
- * See file README.md or go to http://iic.jku.at/eda/research/quantum/ for more information.
+ * See file README.md or go to https://www.cda.cit.tum.de/research/quantum/ for more information.
  */
 
 #include "operations/NonUnitaryOperation.hpp"
 
 #include <algorithm>
+#include <cassert>
 #include <utility>
 
 namespace qc {
     // Measurement constructor
-    NonUnitaryOperation::NonUnitaryOperation(const dd::QubitCount nq, std::vector<dd::Qubit> qubitRegister, std::vector<std::size_t> classicalRegister):
+    NonUnitaryOperation::NonUnitaryOperation(const std::size_t nq, std::vector<Qubit> qubitRegister, std::vector<Bit> classicalRegister):
         qubits(std::move(qubitRegister)), classics(std::move(classicalRegister)) {
         if (qubits.size() != classics.size()) {
             throw std::invalid_argument("Sizes of qubit register and classical register do not match.");
@@ -20,22 +21,22 @@ namespace qc {
         nqubits = nq;
         Operation::setName();
     }
-    NonUnitaryOperation::NonUnitaryOperation(dd::QubitCount nq, dd::Qubit qubit, std::size_t clbit) {
+    NonUnitaryOperation::NonUnitaryOperation(const std::size_t nq, const Qubit qubit, const Bit cbit) {
         type    = Measure;
         nqubits = nq;
         qubits.emplace_back(qubit);
-        classics.emplace_back(clbit);
+        classics.emplace_back(cbit);
         Operation::setName();
     }
 
     // Snapshot constructor
-    NonUnitaryOperation::NonUnitaryOperation(const dd::QubitCount nq, const std::vector<dd::Qubit>& qubitRegister, std::size_t n):
+    NonUnitaryOperation::NonUnitaryOperation(const std::size_t nq, const std::vector<Qubit>& qubitRegister, const std::size_t n):
         NonUnitaryOperation(nq, qubitRegister, Snapshot) {
-        parameter[0] = static_cast<dd::fp>(n);
+        parameter.emplace_back(static_cast<fp>(n));
     }
 
     // General constructor
-    NonUnitaryOperation::NonUnitaryOperation(const dd::QubitCount nq, const std::vector<dd::Qubit>& qubitRegister, OpType op) {
+    NonUnitaryOperation::NonUnitaryOperation(const std::size_t nq, const std::vector<Qubit>& qubitRegister, OpType op) {
         type    = op;
         nqubits = nq;
         targets = qubitRegister;
@@ -43,86 +44,15 @@ namespace qc {
         Operation::setName();
     }
 
-    std::ostream& NonUnitaryOperation::printNonUnitary(std::ostream& os, const std::vector<dd::Qubit>& q, const std::vector<std::size_t>& c, const Permutation& permutation) const {
-        auto qubitIt   = q.cbegin();
-        auto classicIt = c.cbegin();
+    std::ostream& NonUnitaryOperation::printNonUnitary(std::ostream& os, const std::vector<Qubit>& q, const std::vector<Bit>& c, const Permutation& permutation) const {
         switch (type) {
             case Measure:
-                os << name << "\t";
-                if (permutation.empty()) {
-                    for (int i = 0; i < nqubits; ++i) {
-                        if (qubitIt != q.cend() && *qubitIt == i) {
-                            os << "\033[34m" << static_cast<std::size_t>(*classicIt) << "\t"
-                               << "\033[0m";
-                            ++qubitIt;
-                            ++classicIt;
-                        } else {
-                            os << "|\t";
-                        }
-                    }
-                } else {
-                    for (const auto& [physical, logical]: permutation) {
-                        if (qubitIt != q.cend() && *qubitIt == physical) {
-                            os << "\033[34m" << static_cast<std::size_t>(*classicIt) << "\t"
-                               << "\033[0m";
-                            ++qubitIt;
-                            ++classicIt;
-                        } else {
-                            os << "|\t";
-                        }
-                    }
-                }
+                printMeasurement(os, q, c, permutation);
                 break;
             case Reset:
             case Barrier:
             case Snapshot:
-                os << name << "\t";
-                if (permutation.empty()) {
-                    for (int i = 0; i < nqubits; ++i) {
-                        if (qubitIt != q.cend() && *qubitIt == i) {
-                            if (type == Reset) {
-                                os << "\033[31m"
-                                   << "r\t"
-                                   << "\033[0m";
-                            } else if (type == Barrier) {
-                                os << "\033[32m"
-                                   << "b\t"
-                                   << "\033[0m";
-                            } else {
-                                os << "\033[33m"
-                                   << "s\t"
-                                   << "\033[0m";
-                            }
-                            ++qubitIt;
-                        } else {
-                            os << "|\t";
-                        }
-                    }
-                } else {
-                    for (const auto& [physical, logical]: permutation) {
-                        if (qubitIt != q.cend() && *qubitIt == physical) {
-                            if (type == Reset) {
-                                os << "\033[31m"
-                                   << "r\t"
-                                   << "\033[0m";
-                            } else if (type == Barrier) {
-                                os << "\033[32m"
-                                   << "b\t"
-                                   << "\033[0m";
-                            } else {
-                                os << "\033[33m"
-                                   << "s\t"
-                                   << "\033[0m";
-                            }
-                            ++qubitIt;
-                        } else {
-                            os << "|\t";
-                        }
-                    }
-                }
-                if (type == Snapshot) {
-                    os << "\tp: (" << q.size() << ") (" << parameter[1] << ")";
-                }
+                printResetBarrierOrSnapshot(os, q, permutation);
                 break;
             case ShowProbabilities:
                 os << name;
@@ -188,102 +118,17 @@ namespace qc {
         }
     }
 
-    void NonUnitaryOperation::dumpQiskit(std::ostream& of, const RegisterNames& qreg, const RegisterNames& creg, const char*) const {
-        switch (type) {
-            case Measure:
-                if (isWholeQubitRegister(qreg, qubits.front(), qubits.back()) &&
-                    isWholeQubitRegister(qreg, classics.front(), classics.back())) {
-                    of << "qc.measure(" << qreg[qubits.front()].first << ", " << creg[classics.front()].first << ")" << std::endl;
-                } else {
-                    of << "qc.measure([";
-                    for (const auto& q: qubits) {
-                        of << qreg[q].second << ", ";
-                    }
-                    of << "], [";
-                    for (const auto& target: classics) {
-                        of << creg[target].second << ", ";
-                    }
-                    of << "])" << std::endl;
-                }
-                break;
-            case Reset:
-                if (isWholeQubitRegister(qreg, targets.front(), targets.back())) {
-                    of << "append(Reset(), " << qreg[targets.front()].first << ", [])" << std::endl;
-                } else {
-                    of << "append(Reset(), [";
-                    for (const auto& target: targets) {
-                        of << qreg[target].second << ", " << std::endl;
-                    }
-                    of << "], [])" << std::endl;
-                }
-                break;
-            case Snapshot:
-                if (!targets.empty()) {
-                    of << "qc.snapshot(" << parameter[0] << ", qubits=[";
-                    for (const auto& target: targets) {
-                        of << qreg[target].second << ", ";
-                    }
-                    of << "])" << std::endl;
-                }
-                break;
-            case ShowProbabilities:
-                std::cerr << "No equivalent to show_probabilities statement in qiskit" << std::endl;
-                break;
-            case Barrier:
-                if (isWholeQubitRegister(qreg, targets.front(), targets.back())) {
-                    of << "qc.barrier(" << qreg[targets.front()].first << ")" << std::endl;
-                } else {
-                    of << "qc.barrier([";
-                    for (const auto& target: targets) {
-                        of << qreg[target].first << ", ";
-                    }
-                    of << "])" << std::endl;
-                }
-                break;
-            default:
-                std::cerr << "Non-unitary operation with invalid type " << type << " detected. Proceed with caution!" << std::endl;
-                break;
-        }
-    }
-
-    void NonUnitaryOperation::dumpTensor([[maybe_unused]] std::ostream& of, [[maybe_unused]] std::vector<std::size_t>& inds, [[maybe_unused]] std::size_t& gateIdx, [[maybe_unused]] std::unique_ptr<dd::Package>& dd) {
-        if (type == Barrier || type == ShowProbabilities || type == Snapshot)
-            return;
-
-        if (type == Measure) {
-            std::clog << "Skipping measurement in tensor dump." << std::endl;
-        }
-
-        if (type == Reset) {
-            throw QFRException("Reset operation cannot be dumped to tensor");
-        }
-    }
-
-    bool NonUnitaryOperation::actsOn(dd::Qubit i) const {
+    bool NonUnitaryOperation::actsOn(Qubit i) const {
         if (type == Measure) {
             return std::any_of(qubits.cbegin(), qubits.cend(), [&i](const auto& q) { return q == i; });
-        } else if (type == Reset) {
+        }
+        if (type == Reset) {
             return std::any_of(targets.cbegin(), targets.cend(), [&i](const auto& t) { return t == i; });
         }
-        return false; // other non-unitary operations (e.g., barrier statements) may be ignored
+        // other non-unitary operations (e.g., barrier statements) may be ignored
+        return false;
     }
 
-    MatrixDD NonUnitaryOperation::getDD(std::unique_ptr<dd::Package>& dd, [[maybe_unused]] const dd::Controls& controls, [[maybe_unused]] const Targets& targets) const {
-        // these operations do not alter the current state
-        if (type == ShowProbabilities || type == Barrier || type == Snapshot) {
-            return dd->makeIdent(nqubits);
-        }
-
-        throw QFRException("DD for non-unitary operation not available!");
-    }
-    MatrixDD NonUnitaryOperation::getInverseDD(std::unique_ptr<dd::Package>& dd, [[maybe_unused]] const dd::Controls& controls, [[maybe_unused]] const Targets& targets) const {
-        // these operations do not alter the current state
-        if (type == ShowProbabilities || type == Barrier || type == Snapshot) {
-            return dd->makeIdent(nqubits);
-        }
-
-        throw QFRException("Non-unitary operation is not reversible! No inverse DD is available.");
-    }
     bool NonUnitaryOperation::equals(const Operation& op, const Permutation& perm1, const Permutation& perm2) const {
         if (const auto* nonunitary = dynamic_cast<const NonUnitaryOperation*>(&op)) {
             if (getType() != nonunitary->getType()) {
@@ -302,9 +147,9 @@ namespace qc {
                 assert(qubits.size() == classics.size());
                 assert(nonunitary->qubits.size() == nonunitary->classics.size());
 
-                std::set<std::pair<dd::Qubit, std::size_t>> measurements1{};
-                auto                                        qubitIt1   = qubits.cbegin();
-                auto                                        classicIt1 = classics.cbegin();
+                std::set<std::pair<Qubit, Bit>> measurements1{};
+                auto                            qubitIt1   = qubits.cbegin();
+                auto                            classicIt1 = classics.cbegin();
                 while (qubitIt1 != qubits.cend()) {
                     if (perm1.empty()) {
                         measurements1.emplace(*qubitIt1, *classicIt1);
@@ -315,9 +160,9 @@ namespace qc {
                     ++classicIt1;
                 }
 
-                std::set<std::pair<dd::Qubit, std::size_t>> measurements2{};
-                auto                                        qubitIt2   = nonunitary->qubits.cbegin();
-                auto                                        classicIt2 = nonunitary->classics.cbegin();
+                std::set<std::pair<Qubit, Bit>> measurements2{};
+                auto                            qubitIt2   = nonunitary->qubits.cbegin();
+                auto                            classicIt2 = nonunitary->classics.cbegin();
                 while (qubitIt2 != nonunitary->qubits.cend()) {
                     if (perm2.empty()) {
                         measurements2.emplace(*qubitIt2, *classicIt2);
@@ -329,11 +174,95 @@ namespace qc {
                 }
 
                 return measurements1 == measurements2;
-            } else {
-                return Operation::equals(op, perm1, perm2);
+            }
+            return Operation::equals(op, perm1, perm2);
+        }
+        return false;
+    }
+
+    void NonUnitaryOperation::addDepthContribution(std::vector<std::size_t>& depths) const {
+        if (type == Measure || type == Reset) {
+            Operation::addDepthContribution(depths);
+        }
+    }
+
+    void NonUnitaryOperation::printMeasurement(std::ostream& os, const std::vector<Qubit>& q, const std::vector<Bit>& c, const Permutation& permutation) const {
+        auto qubitIt   = q.cbegin();
+        auto classicIt = c.cbegin();
+        os << name << "\t";
+        if (permutation.empty()) {
+            for (std::size_t i = 0; i < nqubits; ++i) {
+                if (qubitIt != q.cend() && *qubitIt == i) {
+                    os << "\033[34m" << *classicIt << "\t"
+                       << "\033[0m";
+                    ++qubitIt;
+                    ++classicIt;
+                } else {
+                    os << "|\t";
+                }
             }
         } else {
-            return false;
+            for (const auto& [physical, logical]: permutation) {
+                if (qubitIt != q.cend() && *qubitIt == physical) {
+                    os << "\033[34m" << *classicIt << "\t"
+                       << "\033[0m";
+                    ++qubitIt;
+                    ++classicIt;
+                } else {
+                    os << "|\t";
+                }
+            }
+        }
+    }
+
+    void NonUnitaryOperation::printResetBarrierOrSnapshot(std::ostream& os, const std::vector<Qubit>& q, const Permutation& permutation) const {
+        auto qubitIt = q.cbegin();
+        os << name << "\t";
+        if (permutation.empty()) {
+            for (std::size_t i = 0; i < nqubits; ++i) {
+                if (qubitIt != q.cend() && *qubitIt == i) {
+                    if (type == Reset) {
+                        os << "\033[31m"
+                           << "r\t"
+                           << "\033[0m";
+                    } else if (type == Barrier) {
+                        os << "\033[32m"
+                           << "b\t"
+                           << "\033[0m";
+                    } else {
+                        os << "\033[33m"
+                           << "s\t"
+                           << "\033[0m";
+                    }
+                    ++qubitIt;
+                } else {
+                    os << "|\t";
+                }
+            }
+        } else {
+            for (const auto& [physical, logical]: permutation) {
+                if (qubitIt != q.cend() && *qubitIt == physical) {
+                    if (type == Reset) {
+                        os << "\033[31m"
+                           << "r\t"
+                           << "\033[0m";
+                    } else if (type == Barrier) {
+                        os << "\033[32m"
+                           << "b\t"
+                           << "\033[0m";
+                    } else {
+                        os << "\033[33m"
+                           << "s\t"
+                           << "\033[0m";
+                    }
+                    ++qubitIt;
+                } else {
+                    os << "|\t";
+                }
+            }
+        }
+        if (type == Snapshot) {
+            os << "\tp: (" << q.size() << ") (" << parameter[1] << ")";
         }
     }
 } // namespace qc
