@@ -1,6 +1,6 @@
 /*
  * This file is part of MQT QFR library which is released under the MIT license.
- * See file README.md or go to http://iic.jku.at/eda/research/quantum/ for more information.
+ * See file README.md or go to https://www.cda.cit.tum.de/research/quantum/ for more information.
  */
 
 #include "operations/Operation.hpp"
@@ -8,101 +8,42 @@
 namespace qc {
     void Operation::setName() {
         switch (type) {
-            case I:
-                strcpy(name, "I   ");
-                break;
-            case H:
-                strcpy(name, "H   ");
-                break;
-            case X:
-                strcpy(name, "X   ");
-                break;
-            case Y:
-                strcpy(name, "Y   ");
-                break;
-            case Z:
-                strcpy(name, "Z   ");
-                break;
-            case S:
-                strcpy(name, "S   ");
-                break;
-            case Sdag:
-                strcpy(name, "Sdag");
-                break;
-            case T:
-                strcpy(name, "T   ");
-                break;
-            case Tdag:
-                strcpy(name, "Tdag");
-                break;
-            case V:
-                strcpy(name, "V   ");
-                break;
-            case Vdag:
-                strcpy(name, "Vdag");
-                break;
-            case U3:
-                strcpy(name, "U   ");
-                break;
-            case U2:
-                strcpy(name, "U2  ");
-                break;
-            case Phase:
-                strcpy(name, "P   ");
-                break;
-            case SX:
-                strcpy(name, "SX  ");
-                break;
-            case SXdag:
-                strcpy(name, "SXdg");
-                break;
-            case RX:
-                strcpy(name, "RX  ");
-                break;
-            case RY:
-                strcpy(name, "RY  ");
-                break;
-            case RZ:
-                strcpy(name, "RZ  ");
-                break;
-            case SWAP:
-                strcpy(name, "SWAP");
-                break;
             case iSWAP:
-                strcpy(name, "iSWP");
+                name = "iSWP";
                 break;
             case Peres:
-                strcpy(name, "Pres");
+                name = "Pr ";
                 break;
             case Peresdag:
-                strcpy(name, "Prdg");
+                name = "Prdg";
                 break;
-            case Compound:
-                strcpy(name, "Comp");
+            case RXX:
+                name = "XX  ";
                 break;
-            case Measure:
-                strcpy(name, "Meas");
+            case RYY:
+                name = "YY  ";
                 break;
-            case Teleportation:
-                strcpy(name, "Tele");
+            case RZZ:
+                name = "ZZ  ";
+                break;
+            case RZX:
+                name = "ZX  ";
+                break;
+            case XXminusYY:
+                name = "XY- ";
+                break;
+            case XXplusYY:
+                name = "XY+ ";
                 break;
             case Reset:
-                strcpy(name, "Rst ");
-                break;
-            case Snapshot:
-                strcpy(name, "Snap");
-                break;
-            case ShowProbabilities:
-                strcpy(name, "Show probabilities");
-                break;
-            case Barrier:
-                strcpy(name, "Barr");
+                name = "Rst ";
                 break;
             case ClassicControlled:
-                strcpy(name, "clc_");
+                name = "clc_";
                 break;
             default:
-                throw QFRException("This constructor shall not be called for gate type (index) " + std::to_string(static_cast<int>(type)));
+                name = toString(type);
+                name.resize(4, ' ');
         }
     }
 
@@ -117,24 +58,26 @@ namespace qc {
         }
 
         bool isZero = true;
-        for (size_t i = 0; i < MAX_PARAMETERS; ++i) {
-            if (parameter[i] != 0.L)
+        for (const auto& p: parameter) {
+            if (p != static_cast<fp>(0)) {
                 isZero = false;
+                break;
+            }
         }
         if (!isZero) {
-            os << "\tp: (";
-            dd::ComplexValue::printFormatted(os, parameter[0]);
-            os << ") ";
-            for (size_t j = 1; j < MAX_PARAMETERS; ++j) {
+            os << "\tp: (" << parameter[0] << ") ";
+            for (size_t j = 1; j < parameter.size(); ++j) {
                 isZero = true;
-                for (size_t i = j; i < MAX_PARAMETERS; ++i) {
-                    if (parameter[i] != 0.L)
+                for (size_t i = j; i < parameter.size(); ++i) {
+                    if (parameter.at(i) != static_cast<fp>(0)) {
                         isZero = false;
+                        break;
+                    }
                 }
-                if (isZero) break;
-                os << "(";
-                dd::ComplexValue::printFormatted(os, parameter[j]);
-                os << ") ";
+                if (isZero) {
+                    break;
+                }
+                os << "(" << parameter.at(j) << ") ";
             }
         }
 
@@ -142,14 +85,14 @@ namespace qc {
     }
 
     std::ostream& Operation::print(std::ostream& os) const {
-        const auto prec_before = std::cout.precision(20);
+        const auto precBefore = std::cout.precision(20);
 
         os << std::setw(4) << name << "\t";
 
         auto controlIt = controls.begin();
         auto targetIt  = targets.begin();
-        for (dd::QubitCount i = 0; i < nqubits; ++i) {
-            if (targetIt != targets.end() && *targetIt == static_cast<dd::Qubit>(i)) {
+        for (std::size_t i = 0; i < nqubits; ++i) {
+            if (targetIt != targets.end() && *targetIt == i) {
                 if (type == ClassicControlled) {
                     os << "\033[1m\033[35m" << name[2] << name[3];
                 } else {
@@ -157,8 +100,8 @@ namespace qc {
                 }
                 os << "\t\033[0m";
                 ++targetIt;
-            } else if (controlIt != controls.end() && controlIt->qubit == static_cast<dd::Qubit>(i)) {
-                if (controlIt->type == dd::Control::Type::pos) {
+            } else if (controlIt != controls.end() && controlIt->qubit == i) {
+                if (controlIt->type == Control::Type::Pos) {
                     os << "\033[32m";
                 } else {
                     os << "\033[31m";
@@ -173,13 +116,13 @@ namespace qc {
 
         printParameters(os);
 
-        std::cout.precision(prec_before);
+        std::cout.precision(precBefore);
 
         return os;
     }
 
     std::ostream& Operation::print(std::ostream& os, const Permutation& permutation) const {
-        const auto prec_before = std::cout.precision(20);
+        const auto precBefore = std::cout.precision(20);
 
         os << std::setw(4) << name << "\t";
         const auto& actualControls = getControls();
@@ -187,18 +130,6 @@ namespace qc {
         auto        controlIt      = actualControls.cbegin();
         auto        targetIt       = actualTargets.cbegin();
         for (const auto& [physical, logical]: permutation) {
-            //            std::cout << static_cast<std::size_t>(physical) << " ";
-            //            if (targetIt != targets.cend())
-            //                std::cout << static_cast<std::size_t>(*targetIt) << " ";
-            //            else
-            //                std::cout << "x ";
-            //
-            //            if (controlIt != controls.cend())
-            //                std::cout << static_cast<std::size_t>(controlIt->qubit) << " ";
-            //            else
-            //                std::cout << "x ";
-            //            std::cout << std::endl;
-
             if (targetIt != actualTargets.cend() && *targetIt == physical) {
                 if (type == ClassicControlled) {
                     os << "\033[1m\033[35m" << name[2] << name[3];
@@ -208,7 +139,7 @@ namespace qc {
                 os << "\t\033[0m";
                 ++targetIt;
             } else if (controlIt != actualControls.cend() && controlIt->qubit == physical) {
-                if (controlIt->type == dd::Control::Type::pos) {
+                if (controlIt->type == Control::Type::Pos) {
                     os << "\033[32m";
                 } else {
                     os << "\033[31m";
@@ -223,7 +154,7 @@ namespace qc {
 
         printParameters(os);
 
-        std::cout.precision(prec_before);
+        std::cout.precision(precBefore);
 
         return os;
     }
@@ -242,38 +173,39 @@ namespace qc {
         }
 
         // check parameters
-        const auto param1 = getParameter();
-        const auto param2 = op.getParameter();
-        for (std::size_t p = 0U; p < qc::MAX_PARAMETERS; ++p) {
-            // it might make sense to use fuzzy comparison here
-            if (param1[p] != param2[p]) { return false; }
+        const auto& param1 = getParameter();
+        const auto& param2 = op.getParameter();
+        if (param1 != param2) {
+            return false;
         }
 
         // check controls
         if (nc1 != 0U) {
-            dd::Controls controls1{};
+            Controls controls1{};
             if (perm1.empty()) {
                 controls1 = getControls();
             } else {
                 for (const auto& control: getControls()) {
-                    controls1.emplace(dd::Control{perm1.at(control.qubit), control.type});
+                    controls1.emplace(Control{perm1.at(control.qubit), control.type});
                 }
             }
 
-            dd::Controls controls2{};
+            Controls controls2{};
             if (perm2.empty()) {
                 controls2 = op.getControls();
             } else {
                 for (const auto& control: op.getControls()) {
-                    controls2.emplace(dd::Control{perm2.at(control.qubit), control.type});
+                    controls2.emplace(Control{perm2.at(control.qubit), control.type});
                 }
             }
 
-            if (controls1 != controls2) { return false; }
+            if (controls1 != controls2) {
+                return false;
+            }
         }
 
         // check targets
-        std::set<dd::Qubit> targets1{};
+        std::set<Qubit> targets1{};
         if (perm1.empty()) {
             targets1 = {getTargets().begin(), getTargets().end()};
         } else {
@@ -282,7 +214,7 @@ namespace qc {
             }
         }
 
-        std::set<dd::Qubit> targets2{};
+        std::set<Qubit> targets2{};
         if (perm2.empty()) {
             targets2 = {op.getTargets().begin(), op.getTargets().end()};
         } else {
@@ -290,10 +222,25 @@ namespace qc {
                 targets2.emplace(perm2.at(target));
             }
         }
-        if (targets1 != targets2) { return false; }
 
-        // operations are identical
-        return true;
+        return targets1 == targets2;
+    }
+
+    void Operation::addDepthContribution(std::vector<std::size_t>& depths) const {
+        std::size_t maxDepth = 0;
+        for (const auto& target: getTargets()) {
+            maxDepth = std::max(maxDepth, depths[target]);
+        }
+        for (const auto& control: getControls()) {
+            maxDepth = std::max(maxDepth, depths[control.qubit]);
+        }
+        maxDepth += 1;
+        for (const auto& target: getTargets()) {
+            depths[target] = maxDepth;
+        }
+        for (const auto& control: getControls()) {
+            depths[control.qubit] = maxDepth;
+        }
     }
 
 } // namespace qc
