@@ -14,7 +14,6 @@ namespace qasm {
      ***/
     void Scanner::nextCh() {
         if (!streams.empty() && streams.top()->eof()) {
-            delete streams.top();
             streams.pop();
             ch   = lines.top().ch;
             line = lines.top().line;
@@ -41,31 +40,34 @@ namespace qasm {
 
     void Scanner::readName(Token& t) {
         std::stringstream ss;
-        while (::isalnum(ch) || ch == '_') {
+        while (::isalnum(ch) != 0 || ch == '_') {
             ss << ch;
             nextCh();
         }
-        t.str   = ss.str();
-        auto it = keywords.find(t.str);
-        t.kind  = (it != keywords.end()) ? it->second : Token::Kind::identifier;
+        t.str = ss.str();
+        if (const auto it = keywords.find(t.str); it != keywords.end()) {
+            t.kind = it->second;
+        } else {
+            t.kind = Token::Kind::Identifier;
+        }
     }
 
     void Scanner::readNumber(Token& t) {
         std::stringstream ss;
-        while (::isdigit(ch)) {
+        while (::isdigit(ch) != 0) {
             ss << ch;
             nextCh();
         }
-        t.kind = Token::Kind::nninteger;
+        t.kind = Token::Kind::Nninteger;
         t.str  = ss.str();
         if (ch != '.') {
             ss >> t.val;
             return;
         }
-        t.kind = Token::Kind::real;
+        t.kind = Token::Kind::Real;
         ss << ch;
         nextCh();
-        while (::isdigit(ch)) {
+        while (::isdigit(ch) != 0) {
             ss << ch;
             nextCh();
         }
@@ -79,7 +81,7 @@ namespace qasm {
             ss << ch;
             nextCh();
         }
-        while (::isdigit(ch)) {
+        while (::isdigit(ch) != 0) {
             ss << ch;
             nextCh();
         }
@@ -93,7 +95,7 @@ namespace qasm {
             nextCh();
         }
         t.str  = ss.str();
-        t.kind = Token::Kind::string;
+        t.kind = Token::Kind::String;
     }
 
     void Scanner::readComment(Token& t) {
@@ -103,51 +105,49 @@ namespace qasm {
             nextCh();
         }
         t.str  = ss.str();
-        t.kind = Token::Kind::comment;
+        t.kind = Token::Kind::Comment;
     }
 
     /***
      * Public Methods
      ***/
-    Scanner::Scanner(std::istream& is):
-        is(is) {
-        keywords["qreg"]               = Token::Kind::qreg;
-        keywords["creg"]               = Token::Kind::creg;
-        keywords["gate"]               = Token::Kind::gate;
-        keywords["measure"]            = Token::Kind::measure;
-        keywords["U"]                  = Token::Kind::ugate;
-        keywords["CX"]                 = Token::Kind::cxgate;
-        keywords["swap"]               = Token::Kind::swap;
-        keywords["mct"]                = Token::Kind::mcx_gray;
-        keywords["mcx"]                = Token::Kind::mcx_gray;
-        keywords["mcx_gray"]           = Token::Kind::mcx_gray;
-        keywords["mcx_recursive"]      = Token::Kind::mcx_recursive;
-        keywords["mcx_vchain"]         = Token::Kind::mcx_vchain;
-        keywords["mcphase"]            = Token::Kind::mcphase;
-        keywords["pi"]                 = Token::Kind::pi;
-        keywords["OPENQASM"]           = Token::Kind::openqasm;
-        keywords["show_probabilities"] = Token::Kind::probabilities;
-        keywords["sin"]                = Token::Kind::sin;
-        keywords["cos"]                = Token::Kind::cos;
-        keywords["tan"]                = Token::Kind::tan;
-        keywords["exp"]                = Token::Kind::exp;
-        keywords["ln"]                 = Token::Kind::ln;
-        keywords["sqrt"]               = Token::Kind::sqrt;
-        keywords["include"]            = Token::Kind::include;
-        keywords["barrier"]            = Token::Kind::barrier;
-        keywords["opaque"]             = Token::Kind::opaque;
-        keywords["if"]                 = Token::Kind::_if;
-        keywords["reset"]              = Token::Kind::reset;
-        keywords["snapshot"]           = Token::Kind::snapshot;
+    Scanner::Scanner(std::istream& in):
+        is(in) {
+        keywords["qreg"]               = Token::Kind::Qreg;
+        keywords["creg"]               = Token::Kind::Creg;
+        keywords["gate"]               = Token::Kind::Gate;
+        keywords["measure"]            = Token::Kind::Measure;
+        keywords["mct"]                = Token::Kind::McxGray;
+        keywords["mcx"]                = Token::Kind::McxGray;
+        keywords["mcx_gray"]           = Token::Kind::McxGray;
+        keywords["mcx_recursive"]      = Token::Kind::McxRecursive;
+        keywords["mcx_vchain"]         = Token::Kind::McxVchain;
+        keywords["mcphase"]            = Token::Kind::Mcphase;
+        keywords["pi"]                 = Token::Kind::Pi;
+        keywords["OPENQASM"]           = Token::Kind::Openqasm;
+        keywords["show_probabilities"] = Token::Kind::Probabilities;
+        keywords["sin"]                = Token::Kind::Sin;
+        keywords["cos"]                = Token::Kind::Cos;
+        keywords["tan"]                = Token::Kind::Tan;
+        keywords["exp"]                = Token::Kind::Exp;
+        keywords["ln"]                 = Token::Kind::Ln;
+        keywords["sqrt"]               = Token::Kind::Sqrt;
+        keywords["include"]            = Token::Kind::Include;
+        keywords["barrier"]            = Token::Kind::Barrier;
+        keywords["opaque"]             = Token::Kind::Opaque;
+        keywords["if"]                 = Token::Kind::If;
+        keywords["reset"]              = Token::Kind::Reset;
+        keywords["snapshot"]           = Token::Kind::Snapshot;
         nextCh();
     }
 
     Token Scanner::next() {
-        while (::isspace(ch)) {
+        // skip over any whitespace
+        while (::isspace(ch) != 0) {
             nextCh();
         }
 
-        Token t = Token(Token::Kind::none, line, col);
+        auto t = Token(Token::Kind::None, line, col);
 
         switch (ch) {
             case 'a':
@@ -202,6 +202,7 @@ namespace qasm {
             case 'X':
             case 'Y':
             case 'Z':
+                // any name specifier [a-zA-Z]
                 readName(t);
                 break;
             case '0':
@@ -215,89 +216,94 @@ namespace qasm {
             case '8':
             case '9':
             case '.':
+                // any number starting with [0-9] or .
                 readNumber(t);
                 break;
             case ';':
-                t.kind = Token::Kind::semicolon;
+                t.kind = Token::Kind::Semicolon;
                 nextCh();
                 break;
             case static_cast<char>(-1):
-                t.kind = Token::Kind::eof;
+                t.kind = Token::Kind::Eof;
                 break;
             case '(':
-                t.kind = Token::Kind::lpar;
+                t.kind = Token::Kind::Lpar;
                 nextCh();
                 break;
             case ')':
-                t.kind = Token::Kind::rpar;
+                t.kind = Token::Kind::Rpar;
                 nextCh();
                 break;
             case '[':
-                t.kind = Token::Kind::lbrack;
+                t.kind = Token::Kind::Lbrack;
                 nextCh();
                 break;
             case ']':
-                t.kind = Token::Kind::rbrack;
+                t.kind = Token::Kind::Rbrack;
                 nextCh();
                 break;
             case '{':
-                t.kind = Token::Kind::lbrace;
+                t.kind = Token::Kind::Lbrace;
                 nextCh();
                 break;
             case '}':
-                t.kind = Token::Kind::rbrace;
+                t.kind = Token::Kind::Rbrace;
                 nextCh();
                 break;
             case ',':
-                t.kind = Token::Kind::comma;
+                t.kind = Token::Kind::Comma;
                 nextCh();
                 break;
             case '+':
                 nextCh();
-                t.kind = Token::Kind::plus;
+                t.kind = Token::Kind::Plus;
                 break;
             case '-':
                 nextCh();
-                t.kind = Token::Kind::minus;
+                t.kind = Token::Kind::Minus;
                 break;
             case '*':
                 nextCh();
-                t.kind = Token::Kind::times;
+                t.kind = Token::Kind::Times;
                 break;
             case '/':
+                // can indicate a comment or a division
                 nextCh();
                 if (ch == '/') {
                     nextCh();
                     readComment(t);
                     nextCh();
                 } else {
-                    t.kind = Token::Kind::div;
+                    t.kind = Token::Kind::Div;
                 }
                 break;
             case '^':
                 nextCh();
-                t.kind = Token::Kind::power;
+                t.kind = Token::Kind::Power;
                 break;
             case '"':
+                // string literal
                 nextCh();
                 readString(t);
                 nextCh();
                 break;
             case '>':
                 nextCh();
-                t.kind = Token::Kind::gt;
+                t.kind = Token::Kind::Gt;
                 break;
             case '=':
+                // must be an equality operator
                 nextCh();
                 if (ch == '=') {
                     nextCh();
-                    t.kind = Token::Kind::eq;
+                    t.kind = Token::Kind::Eq;
                 } else {
-                    std::cerr << "ERROR: UNEXPECTED CHARACTER: '" << ch << "'! " << std::endl;
+                    std::cerr << "ERROR: UNEXPECTED CHARACTER: '" << ch << "'!\n";
                 }
                 break;
             default:
-                std::cerr << "ERROR: UNEXPECTED CHARACTER: '" << ch << "'! " << std::endl;
+                // this should never be reached
+                std::cerr << "ERROR: UNEXPECTED CHARACTER: '" << ch << "'!\n";
                 nextCh();
         }
 
@@ -305,54 +311,20 @@ namespace qasm {
     }
 
     void Scanner::addFileInput(const std::string& filename) {
-        auto* in = new std::ifstream(filename, std::ifstream::in);
+        auto in = std::make_shared<std::ifstream>(filename, std::ifstream::in);
 
         if (in->fail() && filename == "qelib1.inc") {
-            delete in;
-            // internal qelib1.inc
+            // qelib1.inc extensions
+            // all other definitions are redundant as the respective gates are natively supported
             // parser can also read multiple-control versions of each gate
-            auto ss = new std::stringstream{};
-            *ss << "gate u(theta,phi,lambda) q { U(theta,phi,lambda) q; }" << std::endl;
-            *ss << "gate u3(theta,phi,lambda) q { U(theta,phi,lambda) q; }" << std::endl;
-            *ss << "gate u2(phi,lambda) q { U(pi/2,phi,lambda) q; }" << std::endl;
-            *ss << "gate u1(lambda) q { U(0,0,lambda) q; }" << std::endl;
-            *ss << "gate p(lambda) q { U(0,0,lambda) q; }" << std::endl;
-            *ss << "gate cx c, t { CX c, t; }" << std::endl;
-            *ss << "gate id t { U(0,0,0) t; }" << std::endl;
-            *ss << "gate x t { u3(pi,0,pi) t; }" << std::endl;
-            *ss << "gate y t { u3(pi,pi/2,pi/2) t; }" << std::endl;
-            *ss << "gate z t { u1(pi) t; }" << std::endl;
-            *ss << "gate h t { u2(0,pi) t; }" << std::endl;
-            *ss << "gate s t { u1(pi/2) t; }" << std::endl;
-            *ss << "gate sdg t { u1(-pi/2) t; }" << std::endl;
-            *ss << "gate t t { u1(pi/4) t; }" << std::endl;
-            *ss << "gate tdg t { u1(-pi/4) t; }" << std::endl;
-            *ss << "gate rx(theta) t { u3(theta,-pi/2,pi/2) t; }" << std::endl;
-            *ss << "gate ry(theta) t { u3(theta,0,0) t; }" << std::endl;
-            *ss << "gate rz(phi) t { u1(phi) t; }" << std::endl;
-            *ss << "gate sx t { sdg t; h t; sdg t; }" << std::endl;
-            *ss << "gate sxdg t { s t; h t; s t; }" << std::endl;
-            *ss << "gate rxx(theta) a, b { "
-                   "u3(pi/2, theta, 0) a; h b; "
-                   "cx a,b; u1(-theta) b; "
-                   "cx a,b; h b; "
-                   "u2(-pi, pi-theta) a; "
-                   "}"
-                << std::endl;
-            *ss << "gate rzz(theta) a, b { "
-                   "cx a,b; "
-                   "u1(theta) b; "
-                   "cx a,b; "
-                   "}"
-                << std::endl;
+            auto ss = std::make_shared<std::stringstream>();
             *ss << "gate rccx a, b, c { "
                    "u2(0, pi) c; u1(pi/4) c; "
                    "cx b, c; u1(-pi/4) c; "
                    "cx a, c; u1(pi/4) c; "
                    "cx b, c; u1(-pi/4) c; "
                    "u2(0, pi) c; "
-                   "}"
-                << std::endl;
+                   "}\n";
             *ss << "gate rc3x a,b,c,d { "
                    "u2(0,pi) d; u1(pi/4) d; "
                    "cx c,d; u1(-pi/4) d; u2(0,pi) d; "
@@ -363,8 +335,7 @@ namespace qasm {
                    "u2(0,pi) d; u1(pi/4) d; "
                    "cx c,d; u1(-pi/4) d; "
                    "u2(0,pi) d; "
-                   "}"
-                << std::endl;
+                   "}\n";
             *ss << "gate c3x a,b,c,d { "
                    "h d; cu1(-pi/4) a,d; h d; "
                    "cx a,b; "
@@ -379,8 +350,7 @@ namespace qasm {
                    "h d; cu1(pi/4) c,d; h d; "
                    "cx a,c; "
                    "h d; cu1(-pi/4) c,d; h d; "
-                   "}"
-                << std::endl;
+                   "}\n";
             *ss << "gate c3sqrtx a,b,c,d { "
                    "h d; cu1(-pi/8) a,d; h d; "
                    "cx a,b; "
@@ -395,35 +365,25 @@ namespace qasm {
                    "h d; cu1(pi/8) c,d; h d; "
                    "cx a,c; "
                    "h d; cu1(-pi/8) c,d; h d; "
-                   "}"
-                << std::endl;
+                   "}\n";
             *ss << "gate c4x a,b,c,d,e { "
                    "h e; cu1(-pi/2) d,e; h e; "
                    "c3x a,b,c,d; "
                    "h e; cu1(pi/2) d,e; h e; "
                    "c3x a,b,c,d; "
                    "c3sqrtx a,b,c,e; "
-                   "}"
-                << std::endl;
-            *ss << "gate iswap a,b { "
-                   "s a; s b; "
-                   "h a; "
-                   "cx a,b; "
-                   "cx b,a; "
-                   "h b; "
-                   "}"
-                << std::endl;
-
+                   "}\n";
             streams.push(ss);
-            lines.push(LineInfo(ch, line, col));
+            lines.emplace(ch, line, col);
             line = 1;
             col  = 0;
         } else if (in->fail()) {
-            delete in;
+            // file could not be found and it was not the standard include file
             std::cerr << "Failed to open file '" << filename << "'!" << std::endl;
         } else {
+            // file was found and opened
             streams.push(in);
-            lines.push(LineInfo(ch, line, col));
+            lines.emplace(ch, line, col);
             line = 1;
             col  = 0;
         }
